package com.zilber.boot.intelligencesite.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zilber.boot.intelligencesite.entity.IManpower;
import com.zilber.boot.intelligencesite.entity.IProductionPlan;
import com.zilber.boot.intelligencesite.entity.IProgress;
import com.zilber.boot.intelligencesite.entity.IWarn;
import com.zilber.boot.intelligencesite.mapper.IManpowerMapper;
import com.zilber.boot.intelligencesite.mapper.IProductionPlanMapper;
import com.zilber.boot.intelligencesite.mapper.IProgressMapper;
import com.zilber.boot.intelligencesite.mapper.IWarnMapper;
import com.zilber.boot.intelligencesite.service.IIProductionPlanService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zilber.boot.utils.AjaxResult;
import com.zilber.boot.utils.DateUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * <p>
 * 生产计划表 服务实现类
 * </p>
 *
 * @author lsc
 * @since 2025-05-06
 */
@Service
public class IProductionPlanServiceImpl extends ServiceImpl<IProductionPlanMapper, IProductionPlan> implements IIProductionPlanService {

    @Resource
    private IProductionPlanMapper iProductionPlanMapper;
    @Resource
    private IProgressMapper iProgressMapper;
    @Resource
    private IManpowerMapper iManpowerMapper;
    @Resource
    private IWarnMapper iWarnMapper;



    @Override
    public List<IProductionPlan> queryList(IProductionPlan iProductionPlan) {
        return iProductionPlanMapper.queryList(iProductionPlan);
    }

    @Override
    public Map<String, Object> statistics() {
        List<String> recentWeekDates = getRecentWeekDates();
        List<String> namelist=new ArrayList<>();
        List<BigDecimal> planlist=new ArrayList<>();
        List<Long> actuallist=new ArrayList<>();
        for(int i=0;i<recentWeekDates.size();i++){
            List<IProductionPlan> iProductionPlans = iProductionPlanMapper.statistics(recentWeekDates.get(i));
            if(iProductionPlans.size()==0){
                planlist.add(new BigDecimal(0));
                actuallist.add(Long.valueOf(0));
            }

            for (IProductionPlan iProductionPlan : iProductionPlans) {
                //获取每个计划时间差（结束时间-开始时间）
                //long datePoorByDay = DateUtils.getDatePoorByDay(iProductionPlan.getEndTime(), iProductionPlan.getStartTime());
                //获取每个计划每天应完成的百分比
                BigDecimal daypercent = new BigDecimal(1).divide(new BigDecimal(iProductionPlan.getDuration()),2,BigDecimal.ROUND_HALF_UP).multiply(new BigDecimal(100));
                //获取当前日期，减去开始日期，乘以每个计划每天应完成的百分比，算出截止今天应该完成的百分比
                BigDecimal currentpercent = BigDecimal.valueOf(DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getStartTime())).multiply(daypercent);
                if (DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getEndTime()) > 0) {
                    currentpercent = BigDecimal.valueOf(100);
                }
                planlist.add(currentpercent);
                namelist.add(iProductionPlan.getPlanName());
                //查询计划实际填报情况
                List<IProgress> progresseslist = iProgressMapper.queryList(new IProgress().setPlanId(iProductionPlan.getId()));
                long total = progresseslist.stream().mapToLong(IProgress::getDayProgress).sum();
                actuallist.add(total);
            }
        }


        Map<String, Object> result = new HashMap<>();
        result.put("namelist",namelist);
        result.put("datelist",recentWeekDates);
        result.put("planlist",planlist);
        result.put("actuallist",actuallist);
        return result;
    }

    @Override
    public List<IProductionPlan> querywarn(Integer day) {
        List<IProductionPlan> iProductionPlans = iProductionPlanMapper.querywarn();
        List<IProductionPlan> newiProductionPlans=new ArrayList<>();
        for (IProductionPlan iProductionPlan : iProductionPlans) {
            if (DateUtils.getDatePoorByDay(new Date(), iProductionPlan.getEndTime()) > day) {
                newiProductionPlans.add(iProductionPlan);
                IWarn i=iWarnMapper.selectByPlanId(iProductionPlan.getId());
                if(ObjectUtils.isEmpty(i)){
                    IWarn warn=new IWarn();
                    warn.setPlanId(iProductionPlan.getId());
                    warn.setPlanName(iProductionPlan.getPlanName());
                    warn.setCreateTime(new Date());
                    iWarnMapper.insert(warn);
                }
            }
        }
        return newiProductionPlans;
    }

    @Override
    public Map<String, Object> dataOverview() {
        int total=iProductionPlanMapper.queryTotal();//总工期
        int sg=iManpowerMapper.querySg();//施工人员
        int gl=iProductionPlanMapper.queryGl();//管理人员
        List<IProgress> ysg=iProgressMapper.queryYsg();  //已施工
        //int gcjd=iProductionPlanMapper.queryGcjd();//工程进度
        Map map=new HashMap();
        map.put("list",total);//总工期
        map.put("sg",sg);//施工人员
        map.put("gl",gl);//管理人员
        map.put("ysg",ysg.size());// //已施工
        //查询一级计划
        double day=0;
        int totalday=0;List<IProductionPlan> plan = iProductionPlanMapper.bigscreenProgress();
        for(int i=0;i<plan.size();i++){
            if(null!=plan.get(i).getAccumulativeProgress()){
                day+=plan.get(i).getDuration()*plan.get(i).getAccumulativeProgress();
            }else{
                day+=0;

            }
            totalday+=plan.get(i).getDuration();
        }
        map.put("gcjd",day/totalday);// //已施工

        return AjaxResult.success(map);
    }

    @Override
    public List<IProductionPlan> bigscreenProgress() {
       return iProductionPlanMapper.bigscreenProgress();
    }

    //获取最近一周时间
    public static List<String> getRecentWeekDates() {
        List<String> dates = new ArrayList<>();
        DateTimeFormatter formatter = DateTimeFormatter.ofPattern("yyyy-MM-dd");
        LocalDate today = LocalDate.now();

        for (int i = 0; i < 7; i++) {
            LocalDate date = today.minusDays(i);
            dates.add(date.format(formatter));
        }

        return dates;
    }

}
